//
//  EyesView.m
//  YKEyes
//
//  Created by KOYAMA Yoshiaki on Tue Nov 18 2003.
//  Copyright (c) 2003 KOYAMA Yoshiaki. All rights reserved.
//

#import "EyesView.h"
#import "OvalEye.h"


@implementation EyesView

- (id)initWithFrame:(NSRect)frame {
    OvalEye *eye;
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSData *data;
 
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
    }

    data = [defaults objectForKey:@"eyeColorLeft"];
    if (data == nil) {
        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor blackColor]] forKey:@"eyeColorLeft"];
        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor yellowColor]] forKey:@"backColorLeft"];
        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor blackColor]] forKey:@"frameColorLeft"];

        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor blackColor]] forKey:@"eyeColorRight"];
        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor yellowColor]] forKey:@"backColorRight"];
        [defaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor blackColor]] forKey:@"frameColorRight"];

        [defaults synchronize];
    }


    _array = [[NSMutableArray array] retain];

    eye = [[OvalEye alloc] initOvalEye:LEFT_EYE :self];
    [_array addObject:eye];


    eye = [[OvalEye alloc] initOvalEye:RIGHT_EYE :self];
    [_array addObject:eye]; 

    _eyesTimer = [NSTimer scheduledTimerWithTimeInterval:0.30 target:self
     	selector:@selector(eyesTimer:) userInfo:nil repeats:YES];

    return self;
}

// 描画
- (void)drawRect:(NSRect)rect {
    int i;
    OvalEye *eye;
    NSPoint mouseBase;
    NSPoint mouseLocation;
    NSBezierPath *bezier;

    [[NSColor clearColor] set];
    NSRectFill(rect);

/*
    [[NSColor whiteColor] set];
    NSRectFill(rect);
*/

    // マウスの位置を読み込む
    mouseBase = [_eyesWindow mouseLocationOutsideOfEventStream];
    // 座標変換
    mouseLocation = [self convertPoint:mouseBase fromView:nil];


    // 目の位置など計算
    for (i=0; i<[_array count]; i++) {
        eye = [_array objectAtIndex:i];
        [eye setMouse:mouseLocation];
        [eye calcOval];
        [eye calcCenter];
    }
    
    // 目の描画
    for (i=0; i<[_array count]; i++) {
        eye = [_array objectAtIndex:i];
        [eye draw];
    }

    // リサイズエリアを描画
    [[NSColor lightGrayColor] set];
    bezier = [NSBezierPath bezierPath];
    [bezier moveToPoint:NSMakePoint(rect.size.width - 12, 0)];
    [bezier lineToPoint:NSMakePoint(rect.size.width, 12)];
    [bezier lineToPoint:NSMakePoint(rect.size.width, 0)];
    [bezier closePath];
    [bezier fill];
}

// 再描画を要求
- (void)eyesTimer:(id)timer {
    [self setNeedsDisplay:YES];
}

// メモリの解放
- (void)dealloc {
    int i;
    OvalEye *eye;
    
    for (i=0; i<[_array count]; i++) {
        eye = [_array objectAtIndex:i];
        [eye release];
    }

    [_array release];

    [super dealloc];
}

// 起動直後の処理
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    [_eyeColorLeft setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"eyeColorLeft"]]];
    [_backColorLeft setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"backColorLeft"]]];
    [_frameColorLeft setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"frameColorLeft"]]];

    [_eyeColorRight setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"eyeColorRight"]]];
    [_backColorRight setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"backColorRight"]]];
    [_frameColorRight setColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"frameColorRight"]]];

    [_prefsWindow center];

}

// プログラム終了時の処理
- (void)applicationWillTerminate:(NSNotification *)aNotification {
    [self make_data];
}

// 色の保存
- (void)make_data {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_eyeColorLeft color]] forKey:@"eyeColorLeft"];
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_backColorLeft color]] forKey:@"backColorLeft"];
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_frameColorLeft color]] forKey:@"frameColorLeft"];

    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_eyeColorRight color]] forKey:@"eyeColorRight"];
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_backColorRight color]] forKey:@"backColorRight"];
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_frameColorRight color]] forKey:@"frameColorRight"];

    [defaults synchronize];    
}

// カラーの変更
- (IBAction)setColor:(id)sender {
    int tag;
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    // タグでどの色を変更しているか区別している
    tag = [sender tag];
    switch (tag) {
        case 0:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_eyeColorLeft color]] forKey:@"eyeColorLeft"];
        break;
        case 1:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_backColorLeft color]] forKey:@"backColorLeft"];
        break;
        case 2:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_frameColorLeft color]] forKey:@"frameColorLeft"];
        break;
        case 3:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_eyeColorRight color]] forKey:@"eyeColorRight"];
        break;
        case 4:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_backColorRight color]] forKey:@"backColorRight"];
        break;
        case 5:
    [defaults setObject:[NSArchiver archivedDataWithRootObject:[_frameColorRight color]] forKey:@"frameColorRight"];
        break;

    }

    [defaults synchronize];

    [self setNeedsDisplay:YES];
}

// ウインドウを閉じた時の処理
- (BOOL)windowShouldClose:(id)sender {
    [NSApp terminate:self];
    return YES;
}

@end
