//
//  OvalEye.m
//  YKEyes
//
//  Created by KOYAMA Yoshiaki on Tue Nov 18 2003.
//  Copyright (c) 2003 KOYAMA Yoshiaki. All rights reserved.
//

#import "OvalEye.h"
#import "EyesView.h"
#import "math.h"

@implementation OvalEye

- (id)initOvalEye:(EyeType)type :(EyesView *)view {
    self = [super init];
    
    _type = type;
    _mouse = NSMakePoint(0,0);
    _center = _mouse;
    _view = view;

    [self calcOval];
    return self;
}

// マウスの位置
- (void)setMouse:(NSPoint)mouse {
    _mouse = mouse;
}

// 楕円の計算
- (void)calcOval {
    NSRect rect;
    NSPoint oval;

    rect = [_view bounds]; 

    _eyesSpace = rect.size.width * EYES_SPACE / WINDOW_WIDTH;
    oval = NSMakePoint((rect.size.width - _eyesSpace)/4, rect.size.height/2);

    if (_type == LEFT_EYE) {
        _transform.x = oval.x;
    } else {
        _transform.x = oval.x * 3 + _eyesSpace;
    }
    _transform.y = oval.y;

    _ovalAB3 = NSMakePoint(oval.x, oval.y);
    _lineWidthAB = NSMakePoint(_ovalAB3.x * LINE_WIDTH_A / OVAL_MIDDLE_A, _ovalAB3.y * LINE_WIDTH_B / OVAL_MIDDLE_B);
    _ovalAB2 = NSMakePoint(_ovalAB3.x - _lineWidthAB.x, _ovalAB3.y - _lineWidthAB.y);
    _ovalAB = NSMakePoint(_ovalAB3.x * OVAL_A / OVAL_MIDDLE_A, _ovalAB3.y * OVAL_B / OVAL_MIDDLE_B);    
    _eyesSize = NSMakePoint(_ovalAB3.x * EYES_SIZE_A / OVAL_MIDDLE_A, _ovalAB3.y * EYES_SIZE_B / OVAL_MIDDLE_B);
}

//
// マウスの座標 (mx, my)
// 楕円の公式
// (x^2)/(a^2)+(y^2)/(b^2) = 1
// 原点とマウスの座標(mx, my)を通る直線
// y = c*x
//　　上の２つの式の交点 (x, y)
//
- (void)calcCenter {
    float x, y, x1, y1, x2, y2, xx, yy, mx, my, a, b, c;

    a = _ovalAB.x;
    b = _ovalAB.y;

    x1 = _transform.x;
    y1 = _transform.y;

    x2 = _mouse.x;
    y2 = _mouse.y;
    
    mx = x2 - x1;
    my = y2 - y1;

    // mx = 0 の時エラーになるための対処
    if ((-0.0001 < mx) && (mx < 0.0001)) {
        mx = 0.00001;
    }

    c = my/mx;
    
    xx = sqrt(pow(a,2)*pow(b,2)/(pow(b,2)+pow(c,2)*pow(a,2)));
    yy = sqrt(pow(a,2)*pow(b,2)*pow(c,2)/(pow(b,2)+pow(c,2)*pow(a,2)));

//    NSLog(@"x1 y1 x2 y2 %f %f %f %f", x1, y1, x2, y2);
//    NSLog(@"mx my c xx yy %f %f %f %f %f", mx, my, c, xx, yy);

    if (mx > 0) {
        if (my > 0) {
            x = xx;
            y = yy;
        } else {
            x = xx;
            y = -yy;
        }
    } else {
        if (my > 0) {
            x = -xx;
            y = yy;
        } else {
            x = -xx;
            y = -yy;
        }
    }
    

    if ((-xx < mx) && (mx < xx) && (-yy < my) && (my < yy)) {
        _center.x = mx;
        _center.y = my;
    } else {
        _center.x = x;
        _center.y = y;
    }
}

// 描画
- (void)draw {
    NSAffineTransform *at;
    NSBezierPath *bp;
    NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSColor *eyeColor, *backColor, *frameColor;
    
    // 現在の状態を記憶
    [currentContext saveGraphicsState];

    // 座標の中心を設定
    at = [NSAffineTransform transform];
    [at translateXBy:_transform.x yBy:_transform.y];
    [at concat];

    if (_type == LEFT_EYE) {
        frameColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"frameColorLeft"]];
    } else {
        frameColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"frameColorRight"]];
    }
    [frameColor set];
    bp = [NSBezierPath bezierPathWithOvalInRect:
            NSMakeRect(-_ovalAB3.x, -_ovalAB3.y,
            _ovalAB3.x*2,_ovalAB3.y*2)];
    [bp fill];

    if (_type == LEFT_EYE) {
        backColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"backColorLeft"]];
    } else {
        backColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"backColorRight"]];
    }
    [backColor set];
    bp = [NSBezierPath bezierPathWithOvalInRect:
            NSMakeRect(-_ovalAB2.x, -_ovalAB2.y,
            _ovalAB2.x*2,_ovalAB2.y*2)];
    [bp fill];


    if (_type == LEFT_EYE) {
        eyeColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"eyeColorLeft"]];
    } else {
        eyeColor = [NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"eyeColorRight"]];
    }
    [eyeColor set];
    bp = [NSBezierPath bezierPathWithOvalInRect:NSMakeRect(_center.x-_eyesSize.x,_center.y-_eyesSize.y,_eyesSize.x*2,_eyesSize.y*2)];
    [bp fill];

//    [at invert];
//    [at concat];

    // invert を使わないでも元の状態に戻ることができる
    [currentContext restoreGraphicsState];

}

@end
